@@ -360,7 +360,8 @@ gconf_editor_new_key_response (GtkDialog *editor,
 
 		gconf_client_set (gconf_client_get_default (), full_path, value, NULL);
 
-		gconf_value_free (value);
+		if(value)
+		  gconf_value_free (value);
 		g_free (full_path);
 	}
 
@@ -463,8 +464,10 @@ gconf_editor_popup_window_edit_key (gpointer callback_data, guint action, GtkWid
 	
 	editor = gconf_key_editor_new (GCONF_KEY_EDITOR_EDIT_KEY);
 
-	gconf_key_editor_set_value (GCONF_KEY_EDITOR (editor), value);
-	gconf_value_free (value);
+	if(value) {
+	  gconf_key_editor_set_value (GCONF_KEY_EDITOR (editor), value);
+	  gconf_value_free (value);
+	}
 
 	gconf_key_editor_set_key_name (GCONF_KEY_EDITOR (editor), path);
 
@@ -488,6 +491,42 @@ gconf_editor_window_list_view_row_activated (GtkTreeView       *tree_view,
 	gconf_editor_popup_window_edit_key (window, 0, NULL);
 }
 
+static gboolean
+gconf_editor_window_test_expand_row (GtkTreeView       *tree_view,
+				     GtkTreeIter       *iter,
+				     GtkTreePath       *path,
+				     gpointer           data)
+{
+  GConfEditorWindow *gconfwindow = GCONF_EDITOR_WINDOW (data);
+  GdkCursor *cursor;
+
+  if (!GTK_WIDGET_REALIZED (gconfwindow))
+    return FALSE;
+
+  cursor = gdk_cursor_new (GDK_WATCH);
+  gdk_window_set_cursor (GTK_WIDGET (gconfwindow)->window, cursor);
+  gdk_cursor_unref (cursor);
+
+  gdk_display_flush (gtk_widget_get_display (GTK_WIDGET (gconfwindow)));
+
+  return FALSE;
+}
+
+static void
+gconf_editor_window_row_expanded (GtkTreeView       *tree_view,
+				  GtkTreeIter       *iter,
+				  GtkTreePath       *path,
+				  gpointer           data)
+{
+  GConfEditorWindow *gconfwindow = GCONF_EDITOR_WINDOW (data);
+
+  if (!GTK_WIDGET_REALIZED (gconfwindow))
+    return;
+
+  gdk_window_set_cursor (GTK_WIDGET (gconfwindow)->window, NULL);
+  gdk_display_flush (gtk_widget_get_display (GTK_WIDGET (gconfwindow)));
+}
+
 static void
 gconf_editor_popup_window_set_as_default (gpointer callback_data, guint action, GtkWidget *widget)
 {
@@ -585,6 +624,19 @@ const char *image_popup_items_paths [] = {
 	N_("/Unset key"),
 };
 
+static void
+gconf_editor_window_row_activated (GtkTreeView *treeview, 
+				   GtkTreePath *path,
+				   GtkTreeViewColumn *column,
+				   GConfEditorWindow *window)
+{
+	if (gtk_tree_view_row_expanded (treeview, path)) {
+		gtk_tree_view_collapse_row (treeview, path);
+	} else {
+		gtk_tree_view_expand_row (treeview, path, FALSE);
+	}
+}
+
 static void
 gconf_editor_window_selection_changed (GtkTreeSelection *selection, GConfEditorWindow *window)
 {
@@ -1048,6 +1100,15 @@ gconf_editor_window_init (GConfEditorWindow *window)
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (window->tree_view), FALSE);
 	g_signal_connect (G_OBJECT (gtk_tree_view_get_selection (GTK_TREE_VIEW (window->tree_view))), "changed",
 			  G_CALLBACK (gconf_editor_window_selection_changed), window);
+	g_signal_connect (G_OBJECT (window->tree_view), "row-activated", 
+			  G_CALLBACK (gconf_editor_window_row_activated),
+			  window);
+	g_signal_connect (window->tree_view, "test-expand-row",
+			  G_CALLBACK (gconf_editor_window_test_expand_row),
+			  window);
+	g_signal_connect (window->tree_view, "row-expanded",
+			  G_CALLBACK (gconf_editor_window_row_expanded),
+			  window);
 	g_object_unref (G_OBJECT (window->tree_model));
 
 	path = gtk_tree_path_new_first ();
